@@ -10,6 +10,7 @@ import type { GatewayConfig } from './lib/Gateway.ts'
 import Gateway, { GatewayType } from './lib/Gateway.ts'
 import jsonStore from './lib/jsonStore.ts'
 import * as loggers from './lib/logger.ts'
+import { logContainer } from './lib/logger.ts'
 import MqttClient from './lib/MqttClient.ts'
 import SocketManager from './lib/SocketManager.ts'
 import type { CallAPIResult, ZwaveConfig } from './lib/ZwaveClient.ts'
@@ -57,6 +58,7 @@ import { generate } from 'selfsigned'
 import type { ZnifferConfig } from './lib/ZnifferManager.ts'
 import ZnifferManager from './lib/ZnifferManager.ts'
 import { getAllNamedScaleGroups, getAllSensors } from '@zwave-js/core'
+import debugManager from './lib/DebugManager.ts'
 
 const createCertificate = promisify(generate)
 
@@ -267,6 +269,7 @@ export async function startServer(port: number | string, host?: string) {
 	setupInterceptor()
 	await loadSnippets()
 	startZniffer(settings.zniffer)
+	await debugManager.init() // Clean up any old debug temp files
 	await startGateway(settings)
 }
 
@@ -548,6 +551,7 @@ app.use(function (req, res, next) {
 })
 
 app.use(
+	// @ts-expect-error types not matching
 	history({
 		index: '/',
 	}),
@@ -635,6 +639,9 @@ function setupSocket(server: HttpServer) {
 				state.zniffer = zniffer.status()
 			}
 
+			// Add debug session status
+			state.debugCaptureActive = debugManager.isSessionActive()
+
 			cb(state)
 		})
 
@@ -872,6 +879,7 @@ app.get('/api/auth-enabled', apisLimiter, function (req, res) {
 app.post(
 	'/api/authenticate',
 	loginLimiter,
+	// @ts-expect-error types not matching
 	csrfProtection,
 	async function (req, res) {
 		const token = req.body.token
@@ -968,6 +976,7 @@ app.get('/api/logout', apisLimiter, isAuthenticated, function (req, res) {
 app.put(
 	'/api/password',
 	apisLimiter,
+	// @ts-expect-error types not matching
 	csrfProtection,
 	isAuthenticated,
 	async function (req, res) {
@@ -1361,6 +1370,10 @@ app.post(
 
 			restarting = true
 
+			if (debugManager.isSessionActive()) {
+				await debugManager.cancelSession()
+			}
+
 			// Close gateway and restart
 			await gw.close()
 			await destroyPlugins()
@@ -1779,6 +1792,113 @@ app.get('/api/snippet', apisLimiter, async function (req, res) {
 	}
 })
 
+// Debug capture endpoints
+app.get('/api/debug/status', apisLimiter, isAuthenticated, function (req, res) {
+	res.json({
+		success: true,
+		active: debugManager.isSessionActive(),
+	})
+})
+
+app.post(
+	'/api/debug/start',
+	apisLimiter,
+	isAuthenticated,
+	async function (req, res) {
+		try {
+			if (debugManager.isSessionActive()) {
+				return res.json({
+					success: false,
+					message: 'A debug session is already active',
+				})
+			}
+
+			const settings: Settings =
+				jsonStore.get(store.settings) || ({} as Settings)
+			const originalLogLevel = settings.gateway?.logLevel || 'info'
+
+			await debugManager.startSession(gw.zwave, originalLogLevel)
+
+			res.json({
+				success: true,
+				message: 'Debug capture started',
+			})
+		} catch (err) {
+			logger.error('Error starting debug session:', err)
+			res.json({
+				success: false,
+				message: err.message,
+			})
+		}
+	},
+)
+
+app.post(
+	'/api/debug/stop',
+	apisLimiter,
+	isAuthenticated,
+	async function (req, res) {
+		try {
+			if (!debugManager.isSessionActive()) {
+				return res.json({
+					success: false,
+					message: 'No active debug session',
+				})
+			}
+
+			const nodeIds: number[] = req.body.nodeIds || []
+
+			const { archive, cleanup } = await debugManager.stopSession(nodeIds)
+
+			const timestamp = new Date().toISOString().replace(/[:.]/g, '-')
+			res.attachment(`zwave-debug-${timestamp}.zip`)
+			res.setHeader('Content-Type', 'application/zip')
+
+			// Clean up temp files after the archive has been sent
+			archive.on('end', async () => {
+				await cleanup()
+			})
+
+			archive.pipe(res)
+		} catch (err) {
+			logger.error('Error stopping debug session:', err)
+			res.json({
+				success: false,
+				message: err.message,
+			})
+		}
+	},
+)
+
+app.post(
+	'/api/debug/cancel',
+	apisLimiter,
+	isAuthenticated,
+	async function (req, res) {
+		try {
+			if (!debugManager.isSessionActive()) {
+				return res.json({
+					success: false,
+					message: 'No active debug session',
+				})
+			}
+
+			await debugManager.cancelSession()
+
+			res.json({
+				success: true,
+				message: 'Debug capture cancelled',
+			})
+		} catch (err) {
+			logger.error('Error cancelling debug session:', err)
+			res.json({
+				success: false,
+				message: err.message,
+			})
+		}
+	},
+)
+
 // ### ERROR HANDLERS
 
 interface HttpError extends utils.ErrnoException {
