@@ -40,6 +40,7 @@ import { readFile, realpath, readdir, stat, rm, rename, writeFile, lstat, mkdir,
 import { generate } from 'selfsigned';
 import ZnifferManager from "./lib/ZnifferManager.js";
 import { getAllNamedScaleGroups, getAllSensors } from '@zwave-js/core';
+import debugManager from "./lib/DebugManager.js";
 const createCertificate = promisify(generate);
 function multerPromise(m, req, res) {
     return new Promise((resolve, reject) => {
@@ -196,6 +197,7 @@ export async function startServer(port, host) {
     setupInterceptor();
     await loadSnippets();
     startZniffer(settings.zniffer);
+    await debugManager.init(); // Clean up any old debug temp files
     await startGateway(settings);
 }
 const defaultSnippets = [];
@@ -411,7 +413,9 @@ app.use(function (req, res, next) {
         next();
     }
 });
-app.use(history({
+app.use(
+// @ts-expect-error types not matching
+history({
     index: '/',
 }));
 // fix back compatibility with old history mode after switching to hash mode
@@ -481,6 +485,8 @@ function setupSocket(server) {
             if (zniffer) {
                 state.zniffer = zniffer.status();
             }
+            // Add debug session status
+            state.debugCaptureActive = debugManager.isSessionActive();
             cb(state);
         });
         socket.on(inboundEvents.zwave, async (data, cb = noop) => {
@@ -684,7 +690,9 @@ app.get('/api/auth-enabled', apisLimiter, function (req, res) {
     res.json({ success: true, data: isAuthEnabled() });
 });
 // api to authenticate user
-app.post('/api/authenticate', loginLimiter, csrfProtection, async function (req, res) {
+app.post('/api/authenticate', loginLimiter, 
+// @ts-expect-error types not matching
+csrfProtection, async function (req, res) {
     const token = req.body.token;
     let user;
     try {
@@ -754,7 +762,9 @@ app.get('/api/logout', apisLimiter, isAuthenticated, function (req, res) {
     });
 });
 // update user password
-app.put('/api/password', apisLimiter, csrfProtection, isAuthenticated, async function (req, res) {
+app.put('/api/password', apisLimiter, 
+// @ts-expect-error types not matching
+csrfProtection, isAuthenticated, async function (req, res) {
     try {
         const users = jsonStore.get(store.users);
         const user = req.session.user;
@@ -1046,6 +1056,9 @@ app.post('/api/restart', apisLimiter, isAuthenticated, async function (req, res)
         }
         const settings = jsonStore.get(store.settings);
         restarting = true;
+        if (debugManager.isSessionActive()) {
+            await debugManager.cancelSession();
+        }
         // Close gateway and restart
         await gw.close();
         await destroyPlugins();
@@ -1359,6 +1372,86 @@ app.get('/api/snippet', apisLimiter, async function (req, res) {
         res.json({ success: false, message: err.message });
     }
 });
+// Debug capture endpoints
+app.get('/api/debug/status', apisLimiter, isAuthenticated, function (req, res) {
+    res.json({
+        success: true,
+        active: debugManager.isSessionActive(),
+    });
+});
+app.post('/api/debug/start', apisLimiter, isAuthenticated, async function (req, res) {
+    try {
+        if (debugManager.isSessionActive()) {
+            return res.json({
+                success: false,
+                message: 'A debug session is already active',
+            });
+        }
+        const settings = jsonStore.get(store.settings) || {};
+        const originalLogLevel = settings.gateway?.logLevel || 'info';
+        await debugManager.startSession(gw.zwave, originalLogLevel);
+        res.json({
+            success: true,
+            message: 'Debug capture started',
+        });
+    }
+    catch (err) {
+        logger.error('Error starting debug session:', err);
+        res.json({
+            success: false,
+            message: err.message,
+        });
+    }
+});
+app.post('/api/debug/stop', apisLimiter, isAuthenticated, async function (req, res) {
+    try {
+        if (!debugManager.isSessionActive()) {
+            return res.json({
+                success: false,
+                message: 'No active debug session',
+            });
+        }
+        const nodeIds = req.body.nodeIds || [];
+        const { archive, cleanup } = await debugManager.stopSession(nodeIds);
+        const timestamp = new Date().toISOString().replace(/[:.]/g, '-');
+        res.attachment(`zwave-debug-${timestamp}.zip`);
+        res.setHeader('Content-Type', 'application/zip');
+        // Clean up temp files after the archive has been sent
+        archive.on('end', async () => {
+            await cleanup();
+        });
+        archive.pipe(res);
+    }
+    catch (err) {
+        logger.error('Error stopping debug session:', err);
+        res.json({
+            success: false,
+            message: err.message,
+        });
+    }
+});
+app.post('/api/debug/cancel', apisLimiter, isAuthenticated, async function (req, res) {
+    try {
+        if (!debugManager.isSessionActive()) {
+            return res.json({
+                success: false,
+                message: 'No active debug session',
+            });
+        }
+        await debugManager.cancelSession();
+        res.json({
+            success: true,
+            message: 'Debug capture cancelled',
+        });
+    }
+    catch (err) {
+        logger.error('Error cancelling debug session:', err);
+        res.json({
+            success: false,
+            message: err.message,
+        });
+    }
+});
 // catch 404 and forward to error handler
 app.use(function (req, res, next) {
     const err = new Error('Not Found');
